package org.springblade.pson.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.pson.entity.Project;
import org.springblade.pson.service.IProjectService;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 项目表 前端控制器
 * </p>
 *
 * @author zbc
 * @since 2021-06-07
 */
@RestController
@RequestMapping("/project")
@AllArgsConstructor
@Api(value = "项目", tags = "项目管理接口")
public class ProjectController extends BladeController {

	private final IProjectService projectService;


	@PostMapping("/save")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "新增项目")
	public R save(@Valid @RequestBody Project project) {
		project.setCreateTime(new Date());
		return R.status(projectService.save(project));
	}


	@PostMapping("/remove")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "删除项目")
	public R remove(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
		return R.status(projectService.removeByIds(Func.toLongList(ids)));
	}


	@PostMapping("/update")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "修改事件")
	public R update(@Valid @RequestBody Project project) {
		return R.status(projectService.updateById(project));
	}


	@GetMapping("/page")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "分页")
	public R<IPage<Project>> list(Project project, Query query) {
		IPage<Project> pages = projectService.page(Condition.getPage(query), Condition.getQueryWrapper(project));
		return R.data(pages);
	}

	@GetMapping("/list")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "列表")
	public R list() {
		List<Project> list = projectService.list(
			new LambdaQueryWrapper<Project>()
				.isNotNull(Project::getId)
		);
		return R.data(list);
	}


	@GetMapping("/detail")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "详情")
	public R<Project> detail(Project project) {
		Project detail = projectService.getOne(Condition.getQueryWrapper(project));
		return R.data(detail);
	}


	@GetMapping("/select/list")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "项目下拉列表")
	public R selectList() {
		List<Project> list = projectService.list(
			new LambdaQueryWrapper<Project>()
				.isNotNull(Project::getId)
		);
		List<Map<String, Object>> data = list.stream()
			.map(obj -> {
				Map<String, Object> map = new HashMap<>();
				map.put("id", obj.getId());
				map.put("name", obj.getName());
				return map;
			}).collect(Collectors.toList());
		return R.data(data);
	}

}

