package org.springblade.sms.component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;

import org.springblade.sms.bean.dto.AdsMarketTaskInfoLog;
import org.springblade.sms.kafka.PushTaskProducer;
import org.springblade.sms.kafka.SmsStatusKafka;
import org.springblade.sms.service.AdsMarketTaskInfoLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import lombok.extern.slf4j.Slf4j;

/**
 * 短信发送状态服务
 *
 * @Author：william
 * @Date：2022/5/26
 */
@Slf4j
@Service
public class SmsStatusService {
	public final static int SMS_STATUS_SUCCESS = 0;
	public final static int SMS_STATUS_SENDING = 1;
	public final static int SMS_STATUS_FAILURE = 2;

	@Autowired
	private AdsMarketTaskInfoLogService logService;

	@Autowired
	private PushTaskProducer pushTaskProducer;

	@Autowired
	private SmsSender smsSender;

	private final ExecutorService threadPool = Executors.newFixedThreadPool(SmsService.THREAD_POOL_SIZE);

	public void handle(LinkedBlockingQueue<SmsStatusKafka> queue) {
		threadPool.execute(() -> {
			try {
				SmsStatusKafka smsStatusData = queue.take();
				querySmsStatus(smsStatusData);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		});
	}

	private void querySmsStatus(SmsStatusKafka smsStatusData) {
		try {
			// status【0/1/2/3/5】，分别对应状态：（已发送/发送成功/未知结果/点击/失败）
			// 0 已发送
			// 1 发送成功
			// 2 未知结果
			// 3 点击
			// 5 失败
			String mobile = smsStatusData.getMobile();
			String smsBizId = smsStatusData.getSmsBizId();
			Integer times = smsStatusData.getTimes();
			SmsConfig smsConfig = smsStatusData.getSmsConfig();

			int status = smsSender.querySmsStatus(mobile, smsBizId, smsConfig);
			if (status == 0 || status == 1 || status == 3) {
				// 发送成功
				updateSmsPushStatus(smsBizId, SMS_STATUS_SUCCESS);
			} else if (status == 5) {
				// 失败
				updateSmsPushStatus(smsBizId, SMS_STATUS_FAILURE);
			} else {
				// 未知结果
				if (times == 3) {
					updateSmsPushStatus(smsBizId, SMS_STATUS_FAILURE);
				} else {
					smsStatusData.setTimes(times + 1);
					pushTaskProducer.smsStatus(smsStatusData);
				}
			}
		} catch (Exception e) {
			log.error("SmsStatusService:querySmsStatus error is: {}" + e.getMessage(), e);
		}
	}

	// 更新短信发送状态
	private void updateSmsPushStatus(String smsBizId, int status) {
		LambdaQueryWrapper<AdsMarketTaskInfoLog> wrapper = Wrappers.lambdaQuery(AdsMarketTaskInfoLog.class)
				.eq(AdsMarketTaskInfoLog::getSmsBizId, smsBizId);
		AdsMarketTaskInfoLog taskLog = logService.getOne(wrapper);
		if (taskLog != null) {
			taskLog.setPushStatus(status);
			logService.updateById(taskLog);
		}
	}
}
