package org.springblade.sms.controller;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.sms.bean.dto.AdsMarketSmsTemplate;
import org.springblade.sms.component.SmsAttachmentKit;
import org.springblade.sms.enums.CheckStatus;
import org.springblade.sms.service.AdsMarketSmsTemplateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 营销短信模版接口
 * 
 * @author William
 * @date 2022年11月10日 上午11:17:44
 */
@RestController
@RequestMapping("/smsTemplate")
@Api(value = "短信模版接口", tags = { "短信模版接口" })
public class AdsMarketSmsTemplateController extends BladeController {

	@Autowired
	private AdsMarketSmsTemplateService templateService;

	@Value("${blade.file.smsPath}")
	private String uploadDir;
	
	@PostMapping("/upload/attachment")
	@ApiOperation(value = "上传彩信附件", httpMethod = "POST")
	public R<Object> uploadAttachment(@RequestParam MultipartFile file) {
		return SmsAttachmentKit.save(file, uploadDir);
	}

	@PostMapping("/list")
	@ApiOperation(value = "列表")
	public R<IPage<AdsMarketSmsTemplate>> list(AdsMarketSmsTemplate template, String beginTime, String endTime,
			Query query) {
		LambdaQueryWrapper<AdsMarketSmsTemplate> queryWrapper = Wrappers.<AdsMarketSmsTemplate>lambdaQuery();
		if (!StrUtil.isBlankIfStr(beginTime)) {
			queryWrapper.ge(AdsMarketSmsTemplate::getCreateTime, beginTime);
		}
		if (!StrUtil.isBlankIfStr(endTime)) {
			queryWrapper.le(AdsMarketSmsTemplate::getCreateTime, endTime);
		}
		if (!StrUtil.isBlankIfStr(template.getName())) {
			queryWrapper.like(AdsMarketSmsTemplate::getName, template.getName());
		}
		if (template.getSmsType() != null) {
			queryWrapper.eq(AdsMarketSmsTemplate::getSmsType, template.getSmsType());
		}
		if (template.getCheckStatus() != null) {
			queryWrapper.eq(AdsMarketSmsTemplate::getCheckStatus, template.getCheckStatus());
		}
		queryWrapper.orderByDesc(AdsMarketSmsTemplate::getId);
		return R.data(templateService.page(Condition.getPage(query), queryWrapper));
	}

	@PostMapping("/info")
	@ApiOperation(value = "详情")
	public R<AdsMarketSmsTemplate> info(@RequestParam Long id) {
		AdsMarketSmsTemplate info = templateService.getById(id);
		return R.data(info);
	}

	@PostMapping(path = "/add", consumes = "application/json")
	@ApiOperation(value = "添加", httpMethod = "POST")
	public R<String> add(@RequestBody AdsMarketSmsTemplate template) {
		return templateService.insertOrUpdate(template);
	}

	@PostMapping("/update")
	@ApiOperation(value = "更新", httpMethod = "POST")
	public R<String> update(@RequestBody AdsMarketSmsTemplate template) {
		if (template == null || template.getId() == null || template.getId() <= 0) {
			return R.fail("参数错误，修改失败");
		}

		if (template.getCheckStatus() != CheckStatus.no.getValue()) {
			return R.fail("只能修改未通过审核的模版");
		}

		return templateService.insertOrUpdate(template);
	}

	@PostMapping("/delete")
	@ApiOperation(value = "删除", httpMethod = "POST")
	public R<String> delete(@RequestParam Long id) {
		AdsMarketSmsTemplate template = templateService.getById(id);
		if (template.getCheckStatus() == CheckStatus.waiting.getValue()) {
			return R.fail("正在审核，不支持删除");
		}

		return R.status(templateService.removeById(id));
	}

}