package org.springblade.sms.service;

import java.io.File;
import java.nio.charset.Charset;
import java.util.Date;

import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.sms.bean.dto.AdsMarketSmsTemplate;
import org.springblade.sms.bean.params.TemplateApplyResponse;
import org.springblade.sms.component.SmsConfig;
import org.springblade.sms.component.SmsSender;
import org.springblade.sms.component.SmsService;
import org.springblade.sms.enums.CheckStatus;
import org.springblade.sms.enums.SmsType;
import org.springblade.sms.mapper.AdsMarketSmsTemplateMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.util.HtmlUtils;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.xiaoymin.knife4j.core.util.StrUtil;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.io.FileUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 短信模版
 * 
 * @author William
 * @date 2022年11月11日 上午12:35:20
 */
@Slf4j
@Service
@DS("user_analysis_db")
public class AdsMarketSmsTemplateService extends ServiceImpl<AdsMarketSmsTemplateMapper, AdsMarketSmsTemplate> {

	@Autowired
	private SmsSender smsSender;

	@Value("${blade.file.smsPath}")
	private String uploadDir;

	/**
	 * 新增或者更新
	 * 
	 * @param template
	 * @return
	 */
	public R<String> insertOrUpdate(AdsMarketSmsTemplate template) {
		template.setContent(HtmlUtils.htmlUnescape(template.getContent()));

		template.setCreateTime(new Date());
		template.setCreateUserId(SecureUtil.getUserId());
		template.setCreateUserName(SecureUtil.getUserName());

		SmsType smsType = SmsType.reslove(template.getSmsType());
		SmsConfig smsConfig = SmsConfig.of(smsType);
		template.setSmsId(smsConfig.getSmsid());

		if (smsType != SmsType.digital) {
			// 通知类型的短信是已经分配好的短信模板，不需要申请，直接通过
			template.setSmsSign(SmsService.SMS_SIGN_FOR_NORMAL);// 固定字符串
			final String contentId = SmsService.CONTENT_ID_FOR_NORMAL;
			template.setContentId(contentId);

			template.setCheckTime(new Date());
			template.setCheckStatus(CheckStatus.ok.getValue());
			template.setCheckRemark("审核成功");
		} else {
			// 调用申请数字短信模板
			String contentOld = template.getContent();
			// 需要在短信模板内容中添加短信签名变量
			template.setContent("【${projects}】" + contentOld);
			String base64Content = buildTemplateContent(template);
			TemplateApplyResponse tar = smsSender.updtempsms(template.getName(), base64Content, smsConfig);
			if (tar == null) {
				return R.fail("短信模板申请接口调用失败");
			}

			if (!tar.isSuccess()) {
				// 申请失败
				String errorInfo = tar.getErrorinfo();
				boolean errorInfoEmpty = StrUtil.isBlank(errorInfo);
				if (errorInfoEmpty) {
					//解析错误信息失败，很可能是ip没有在白名单
					log.error(String.format("AdsMarketSmsTemplateService:template apply request|response=%s", tar.getResponse()));
					JSONObject jo = JSON.parseObject(tar.getResponse());
					errorInfo = jo.getString("data");
				}
				return R.fail(errorInfo);
			}

			template.setContentId(tar.getData().getMmsid());
			template.setCheckStatus(CheckStatus.waiting.getValue());
		}

		return R.status(saveOrUpdate(template));
	}

	/**
	 * 生成申请短信模板需要的内容参数
	 * 
	 * @param template
	 * @return
	 */
	private String buildTemplateContent(AdsMarketSmsTemplate template) {
		// 类型与内容之间要用|分开，各个类型之间要用英文字符逗号隔开
		StringBuffer content = new StringBuffer();
		String txtBase64 = Base64.encode(template.getContent().getBytes(Charset.forName("gb2312")));
		content.append(",txt" + txtBase64);
		String attachment = template.getAttachment();
		if (StrUtil.isNotBlank(attachment)) {
			// 短信模板包含文件
			String filePath = uploadDir + template.getAttachment();
			if (FileUtil.exist(filePath)) {
				String extName = FileUtil.extName(filePath).toLowerCase();
				String fileBase64 = Base64.encode(new File(filePath));
				content.append("," + extName + fileBase64);
			}
		}
		return content.toString();
	}
}