package org.springblade.sms.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import org.springblade.sms.bean.dto.AdsMarketTaskInfoChecker;
import org.springblade.sms.enums.CheckStatus;
import org.springblade.sms.mapper.AdsMarketTaskInfoCheckerMapper;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Author：william
 * @Date：2022/6/8
 */
@Service
@DS("user_analysis_db")
public class AdsMarketTaskInfoCheckerService extends ServiceImpl<AdsMarketTaskInfoCheckerMapper, AdsMarketTaskInfoChecker> {

	/**
	 * 所有审核员是否全部审核完成
	 *
	 * @param taskId
	 * @return
	 */
	public boolean isAllChecked(Long taskId) {
		List<AdsMarketTaskInfoChecker> list = getTaskCheckerList(taskId);
		if (CollUtil.isNotEmpty(list)) {
			boolean flag = true;
			for (AdsMarketTaskInfoChecker item : list) {
				int status = item.getStatus();
				if (status == CheckStatus.no.getValue()) {
					//有一个未通过就视为审核完成，直接作废
					return true;
				}

				if (status == CheckStatus.waiting.getValue()) {
					//没有作废，有一个waiting就视为没有审核完成
					flag = false;
				}
			}

			return flag;
		}
		return false;
	}

	/**
	 * 任务是否通过
	 *
	 * @param taskId
	 * @return
	 */
	public boolean isCheckOk(Long taskId) {
		List<AdsMarketTaskInfoChecker> list = getTaskCheckerList(taskId);
		if (CollUtil.isNotEmpty(list)) {
			int yesCount = 0;
			for (AdsMarketTaskInfoChecker item : list) {
				int status = item.getStatus();
				if (status == CheckStatus.ok.getValue()) {
					yesCount += 1;
				}
			}

			return yesCount == list.size();
		}
		return true;
	}

	/**
	 * 根据任务查询所有审核人
	 *
	 * @param taskId
	 * @return
	 */
	public List<AdsMarketTaskInfoChecker> getTaskCheckerList(Long taskId) {
		LambdaQueryWrapper<AdsMarketTaskInfoChecker> wrapper = Wrappers.lambdaQuery();
		wrapper.eq(AdsMarketTaskInfoChecker::getTaskId, taskId);
		return list(wrapper);
	}

	/**
	 * 清空某个任务的审核人
	 *
	 * @param taskId
	 * @return
	 */
	public boolean clear(Long taskId) {
		LambdaQueryWrapper<AdsMarketTaskInfoChecker> wrapper = Wrappers.lambdaQuery();
		wrapper.eq(AdsMarketTaskInfoChecker::getTaskId, taskId);
		return remove(wrapper);
	}

	/**
	 * 批量插入某个任务的审核人
	 *
	 * @param checkerList
	 * @return
	 */
	public boolean saveList(List<AdsMarketTaskInfoChecker> checkerList) {
		if (checkerList != null && !checkerList.isEmpty()) {
			return saveBatch(checkerList);
		}
		return false;
	}

	/**
	 * 检查某个审核人是否有某个任务的审核权限
	 *
	 * @param taskId
	 * @param checkUserId
	 * @return
	 */
	public boolean exist(Long taskId, String checkUserId) {
		return getChecker(taskId, checkUserId) != null;
	}

	/**
	 * 某个人对某个任务是否已经审核过
	 *
	 * @param taskId
	 * @param checkUserId
	 * @return
	 */
	public boolean isChecked(Long taskId, String checkUserId) {
		AdsMarketTaskInfoChecker checker = getChecker(taskId, checkUserId);
		return checker != null && checker.getStatus() != CheckStatus.waiting.getValue();
	}

	public AdsMarketTaskInfoChecker getChecker(Long taskId, String checkUserId) {
		LambdaQueryWrapper<AdsMarketTaskInfoChecker> wrapper = Wrappers.lambdaQuery();
		wrapper.eq(AdsMarketTaskInfoChecker::getTaskId, taskId);
		wrapper.eq(AdsMarketTaskInfoChecker::getCheckUserId, checkUserId);
		AdsMarketTaskInfoChecker checker = getOne(wrapper);
		return checker;
	}
}
