package org.springblade.sms.service;

import java.util.Date;

import org.springblade.sms.bean.dto.AdsMarketTaskInfo;
import org.springblade.sms.bean.dto.AdsMarketTaskInfoLog;
import org.springblade.sms.component.SmsStatusService;
import org.springblade.sms.mapper.AdsMarketTaskInfoLogMapper;
import org.springframework.stereotype.Service;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 短信任务日志
 * 
 * @author William
 * @date 2022年11月11日 上午12:41:24
 */
@Service
@DS("user_analysis_db")
public class AdsMarketTaskInfoLogService extends ServiceImpl<AdsMarketTaskInfoLogMapper, AdsMarketTaskInfoLog> {

	/**
	 * 保存短信发送记录
	 *
	 * @param task 对应的任务
	 * @param messageInfo 实际短信内容
	 * @param phone 手机号
	 * @param smsJson 短信参数
	 * @param smsTemplateId 短信模板ID
	 * @param smsBizId 联通返回的业务，用于查询短信真实发送状态
	 * @return
	 */
	public boolean saveSmsLog(AdsMarketTaskInfo task, String messageInfo, String phone, String smsJson,
							  String smsTemplateId, String smsBizId) {
		AdsMarketTaskInfoLog taskLog = new AdsMarketTaskInfoLog();
		taskLog.setTaskId(task.getTaskId());
		taskLog.setMissionName(task.getMissionName());
		taskLog.setGroupId(task.getGroupId());
		taskLog.setGroupName(task.getGroupName());
		taskLog.setMessageInfo(messageInfo);
		taskLog.setChannelId(task.getChannelId());
		taskLog.setMarketType(1);
		taskLog.setAddTime(new Date());
		taskLog.setPushTime(new Date());
		taskLog.setPushStatus(SmsStatusService.SMS_STATUS_SENDING);

		taskLog.setPhone(phone);

		taskLog.setSmsJson(smsJson);
		taskLog.setSmsTemplateCode(smsTemplateId);
		taskLog.setSmsBizId(smsBizId);

		return save(taskLog);
	}

//	/**
//	 * 保存邮件发送记录
//	 *
//	 * @param task
//	 * @param messageInfo
//	 * @param phone
//	 * @param email
//	 * @return
//	 */
//	public boolean saveEmailLog(AdsMarketTaskInfo task, String messageInfo, String phone, String email) {
//		AdsMarketTaskInfoLog taskLog = new AdsMarketTaskInfoLog();
//		taskLog.setTaskId(task.getTaskId());
//		taskLog.setMissionName(task.getMissionName());
//		taskLog.setGroupId(task.getGroupId());
//		taskLog.setGroupName(task.getGroupName());
//		taskLog.setMessageInfo(messageInfo);
//		taskLog.setChannelId(task.getChannelId());
//		taskLog.setMarketType(1);
//		taskLog.setAddTime(new Date());
//		taskLog.setPushTime(new Date());
//		taskLog.setPushStatus(SmsStatusService.SMS_STATUS_SUCCESS);
//
//		taskLog.setPhone(phone);
//
//		taskLog.setEmailTitle("");
//		taskLog.setEmail(email);
//
//		return save(taskLog);
//	}
}
