package org.springblade.sms.service;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.sms.bean.dto.AdsMarketTaskInfo;
import org.springblade.sms.bean.dto.AdsMarketTaskInfoChecker;
import org.springblade.sms.bean.dto.DefineGroup;
import org.springblade.sms.bean.params.TaskAddParam;
import org.springblade.sms.enums.CheckStatus;
import org.springblade.sms.enums.CycleType;
import org.springblade.sms.enums.GroupType;
import org.springblade.sms.enums.SmsPushPattern;
import org.springblade.sms.enums.SmsPushStatus;
import org.springblade.sms.enums.TaskDraftStatus;
import org.springblade.sms.mapper.AdsMarketTaskInfoMapper;
import org.springblade.useranalysis.entity.UserExcelImportLog;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.service.UserGroupService;
import org.springblade.useranalysis.service.normal.UserExcelImportLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.StrUtil;

/**
 * 营销任务
 * 
 * @author William
 * @date 2022年11月11日 上午12:41:40
 */
@Service
@DS("user_analysis_db")
public class AdsMarketTaskInfoService extends ServiceImpl<AdsMarketTaskInfoMapper, AdsMarketTaskInfo> {
	@Autowired
	private UserGroupService userGroupService;

	@Autowired
	private UserExcelImportLogService importLogService;

	@Autowired
	private DefineGroupService defineGroupService;

	@Autowired
	private AdsMarketTaskInfoCheckerService checkerService;

	public List<String> getSortNoList() {
		return baseMapper.getSortNoList();
	}

	public Kv getPageList(StringBuffer join, StringBuffer where, Integer current, Integer size) {
		StringBuffer sql = new StringBuffer();
		sql.append(join);
		sql.append(where);

		int total = baseMapper.getPageListCount(sql.toString());
		int pages = 0;
		if (size >= total) {
			pages = 1;
		} else {
			pages = total / size;
			if (total % size > 0) {
				pages += 1;
			}
		}

		sql.append(" order by t.add_time desc");
		sql.append(" limit " + (current - 1) * size + "," + size + "");

		Kv data = Kv.create();
		data.set("current", current);
		data.set("size", size);
		data.set("total", total);
		data.set("pages", pages);
		data.set("list", baseMapper.getPageList(sql.toString()));
		return data;
	}

	public R<Object> insertOrUpdate(TaskAddParam data) {
		AdsMarketTaskInfo task = data.getTaskInfo();
		Long taskId = task.getTaskId();
		if (taskId != null) {
			// 更新短信任务
			AdsMarketTaskInfo old = getById(taskId);
			if (old.getCheckStatus() == CheckStatus.no.getValue()) {
				// 已作废的任务不允许再次修改
				return R.fail("已作废的任务不允许编辑");
			}
			if (old.getDraftStaus() == TaskDraftStatus.not_draft.getValue()) {
				// 非草稿任务不允许编辑
				return R.fail("非草稿任务不允许编辑");
			}
		}

		List<AdsMarketTaskInfoChecker> checkerList = data.getCheckerList();
		if (CollUtil.isEmpty(checkerList)) {
			return R.fail("未添加审核人");
		}

		SmsPushPattern pushPattern = SmsPushPattern.reslove(task.getPushStatus());
		if (pushPattern == SmsPushPattern.atTime) {
			// 定时发送
			if (task.getPushTime() == null) {
				return R.fail("定时推送模式下，发送时间不可为空");
			}

			LocalDateTime pushTime = LocalDateTimeUtil.of(task.getPushTime());
			LocalDateTime now = LocalDateTimeUtil.of(new Date());
			if (pushTime.isBefore(now)) {
				return R.fail("推送时间必须大于当前时间");
			}
		}

		if (pushPattern == SmsPushPattern.cycleTime) {
			if (task.getCycleStartTime() == null || task.getCycleEndTime() == null) {
				return R.fail("周期性推送，开始时间和截止时间不可为空");
			}

			LocalDateTime start = LocalDateTimeUtil.of(task.getCycleStartTime());
			LocalDateTime end = LocalDateTimeUtil.of(task.getCycleEndTime());
			if (start.isBefore(end)) {
				return R.fail("截止时间必须大于开始时间");
			}

			if (task.getCycleSendTime() == null) {
				return R.fail("周期性推送，发送时间不可为空");
			}

			CycleType cycleType = CycleType.reslove(task.getCycleType());
			if (cycleType == CycleType.week && task.getCycleWeek() == null) {
				return R.fail("按周发送时，周几不可为空");
			}
			if (cycleType == CycleType.month && task.getCycleDayOfMonth() == null) {
				return R.fail("按月发送时，每月几号不可为空");
			}

			if (cycleType == CycleType.day) {
				task.setCycleWeek(null);
				task.setCycleDayOfMonth(null);
			}

			if (cycleType == CycleType.week) {
				task.setCycleDayOfMonth(null);
			}

			if (cycleType == CycleType.month) {
				task.setCycleWeek(null);
			}
		}

		task.setCreater(SecureUtil.getUserName());
		task.setAddTime(new Date());
		task.setDraftStaus(TaskDraftStatus.draft.getValue());
		task.setCheckStatus(CheckStatus.waiting.getValue());
		task.setPushStatus(SmsPushStatus.waiting.getValue());

		long count = 0;
		GroupType groupType = GroupType.reslove(task.getGroupType());
		if (groupType == GroupType.import_group) {
			UserExcelImportLog importLog = importLogService.getById(task.getGroupId());
			count = importLog != null ? importLog.getCount() : 0;
		} else if (groupType == GroupType.tag_group) {
			UserGroupEntity userGroupEntity = userGroupService.getById(task.getGroupId());
			if (userGroupEntity != null && !StrUtil.isEmpty(userGroupEntity.getRules())) {
				String reachCount = userGroupEntity.getReachCount();
				try {
					count = Long.parseLong(reachCount);
				} catch (Exception e) {
				}
			}
		} else if (groupType == GroupType.custom_group) {
			// 自定义人群
			DefineGroup group = defineGroupService.getById(task.getGroupId());
			Long amount = group.getAmount();
			count = amount != null ? amount.longValue() : 0;
		}

		task.setTargetGroupNumber(count);

		if (taskId != null) {
			// 更新短信任务
			if (CollUtil.isNotEmpty(checkerList)) {
				checkerService.clear(taskId);
			}
		}

		boolean ret = saveOrUpdate(task);
		checkerList.forEach(ch -> {
			ch.setTaskId(task.getTaskId());
		});
		ret &= checkerService.saveList(checkerList);
		return R.status(ret);
	}
}
