package org.springblade.useranalysis.activespid;

import java.util.Date;
import java.util.List;

import org.springblade.sms.component.SmsSender;
import org.springblade.sms.kafka.PushTaskProducer;
import org.springblade.useranalysis.activespid.common.DateTimeKit;
import org.springblade.useranalysis.activespid.common.DateTimeType;
import org.springblade.useranalysis.activespid.entity.SpidSmsActiveLog;
import org.springblade.useranalysis.activespid.service.ActiveClientService;
import org.springblade.useranalysis.activespid.service.SpidSmsActiveLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import cn.hutool.core.collection.CollUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 产品促活定时任务
 * 
 * @author william
 * @date 2022年9月7日 下午3:13:02
 */
@Slf4j
@Configuration
@EnableScheduling
public class ActiveSpidJob {
	@Autowired
	private ActiveClientService clientService;

	@Autowired
	private SpidSmsActiveLogService spidSmsLogService;

	@Autowired
	private PushTaskProducer pushTaskProducer;

	@Autowired
	private SmsSender smsSender;

	/**
	 * 查询零流量用户，每天上午10点检测
	 */
	@Scheduled(cron = "0 0 10 * * *")
	public void findZeroFlowUser() {
		log.info("ActiveSpidJob is started");
		// windata测试后，打开该代码段
		if (!smsSender.isProd()) {
			return;
		}

		String spid = "1051";
		// 查询零流量用户
		List<String> phoneList = clientService.getNoFlowUserIdList();
		if (CollUtil.isEmpty(phoneList)) {
			return;
		}

		for (int i = 0, size = phoneList.size(); i < size; i++) {
			String mobile = phoneList.get(i);
			if (i % 2 != 0) {
				pushTaskProducer.smsSpid(mobile, spid);
			} else {
				Integer firstDay = DateTimeKit.getDateValue(DateTimeType.DAY, DateTimeKit.getPreDay(3));
				SpidSmsActiveLog data = new SpidSmsActiveLog();
				data.setActivityName("芒果合约包产品促活的需求开发");
				data.setApiName("");
				data.setBatchNo(String.valueOf(firstDay));
				data.setCategory("B");
				data.setSendTime(new Date());
				data.setSmsContent("");
				data.setSmsTaskId("");
				data.setSpid(spid);
				data.setSpname("芒果19元产品（含合约）专用");
				data.setTaskPerson("杜伟");
				data.setUserId(mobile);
				spidSmsLogService.save(data);
			}
		}
	}

}
