package org.springblade.useranalysis.common.utils;

import java.util.ArrayList;
import java.util.List;

import cn.hutool.core.util.PhoneUtil;
import org.springblade.useranalysis.entity.clickhouse.UserExcelImportPhone;
import org.springblade.useranalysis.service.normal.UserExcelImportPhoneService;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;

//@Slf4j
public class UserExcelImportListener extends AnalysisEventListener<UserExcelImportPhone> {
	private static final int BATCH_COUNT = 10000;

	private UserExcelImportPhoneService importPhoneService;
	private long total;
	private Long id;

	private List<UserExcelImportPhone> list = new ArrayList<>();

	public UserExcelImportListener(Long id, UserExcelImportPhoneService importPhoneService) {
		this.id = id;
		this.importPhoneService = importPhoneService;
	}

	@Override
	public void invoke(UserExcelImportPhone data, AnalysisContext analysisContext) {
//		log.info("解析到一条数据:{}", JSON.toJSONString(data));
		if(!PhoneUtil.isMobile(data.getPhone())){
			//过滤非手机号
			return;
		}
		data.setId(String.valueOf(id));
		list.add(data);
		// 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
		if (list.size() >= BATCH_COUNT) {
			saveData();
			// 存储完成清理 list
			list.clear();
		}
	}

	@Override
	public void doAfterAllAnalysed(AnalysisContext analysisContext) {
		saveData();
//		log.info("所有数据解析完成！");
	}

	/**
	 * 将数据存储数据库
	 */
	private void saveData() {
//		log.info("{}条数据，开始存储数据库！", list.size());
		importPhoneService.saveBatch(list);
		total += list.size();
//		log.info("存储数据库成功！");
	}

	public long getTotal() {
		return total;
	}
}
