package org.springblade.useranalysis.controller;

import java.io.File;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.useranalysis.common.utils.Constants;
import org.springblade.useranalysis.dto.ResultSegment;
import org.springblade.useranalysis.dto.param.ExcelExportPageParam;
import org.springblade.useranalysis.dto.param.ExcelExportParam;
import org.springblade.useranalysis.entity.MidCpidUserId;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.entity.UserProfileExcelLog;
import org.springblade.useranalysis.entity.clickhouse.ZUserTagEntity;
import org.springblade.useranalysis.service.UserGroupService;
import org.springblade.useranalysis.service.normal.ExcelService;
import org.springblade.useranalysis.service.normal.MidCpidUserIdService;
import org.springblade.useranalysis.service.normal.UserGroupRuleService;
import org.springblade.useranalysis.service.normal.UserProfileExcelLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.util.FileUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

/**
 * 导出用户信息到excel
 *
 * @author william
 *
 */
@Slf4j
@RestController
@RequestMapping("/v1/user/excel")
@Api(value = "用户标签导出", tags = "用户标签导出")
public class UserExcelController extends BladeController {
	@Value("${blade.file.excelPath:}")
	private String excelPath;

	@Autowired
	private UserGroupService userGroupService;

	@Autowired
	private UserProfileExcelLogService excelLogService;

	@Autowired
	private ExcelService excelService;

	@Autowired
	private MidCpidUserIdService cpidUserIdService;

	@Autowired
	private UserGroupRuleService userGroupRuleService;

	@ApiOperation("导出Excel")
	@RequestMapping(value = "/export", method = RequestMethod.POST)
	public R<Object> export(@RequestBody ExcelExportParam params) {
		if (params == null || params.getCount() == null || params.getCount() == 0) {
			return R.fail("缺少参数");
		}

		if (params.getCount() > 1000000) {
			log.info("最大导出条数不可超过100万");
			return R.fail("最大导出条数不可超过100万");
		}

		String sysUserName = "管理员";
		Long sysUserId = 0l;

		BladeUser user = getUser();
		if (user != null) {
			sysUserName = getUser().getNickName();
			sysUserId = getUser().getUserId();
		}

		File dir = new File(excelPath);
		dir.mkdirs();

		UserGroupEntity userGroupEntity = userGroupService.getById(params.getGroupId());
		if (userGroupEntity == null) {
			return R.fail("人群不存在");
		}

		String groupName = userGroupEntity.getGroupName();
		if (StringUtils.isNotBlank(params.getMonthStart()) && StringUtils.isNotBlank(params.getMonthEnd())) {
			groupName = groupName + "（" + params.getMonthStart() + "-" + params.getMonthEnd() + "）";
			//此种情况没有计算总数，所以暂定100万条
			params.setCount(1000000);
		}
		UserProfileExcelLog excelLog = new UserProfileExcelLog();
		excelLog.setId(IdWorker.getId());
		excelLog.setCount(params.getCount());
		excelLog.setCreateTime(new Date());
		excelLog.setFileName(excelLog.getId() + ".xlsx");
		excelLog.setFilePath(excelPath + excelLog.getFileName());
		excelLog.setGroupId(params.getGroupId());
		excelLog.setGroupName(groupName);
		excelLog.setStatus(UserProfileExcelLogService.STATUS_DOING);
		excelLog.setSysUserId(sysUserId);
		excelLog.setSysUserName(sysUserName);
		excelLog.setRemark(params.isDesens() ? "脱敏" : "不脱敏");

		return handleExport(excelLog, userGroupEntity, params);
	}

	/**
	 * 自动计算并导出前后几个月的画像数据
	 *
	 * @param params
	 * @return
	 */
//	private R<Object> handleExportByMonths(UserProfileExcelLog excelLog, UserGroupEntity userGroupEntity,
//			ExcelExportParam params) {
//		log.info("基于导入的人群导出前后几个月的数据");
//		// 基于导入的人群导出前后几个月的数据
//		String groupName = excelLog.getGroupName() + "（" + params.getMonthStart() + "-" + params.getMonthEnd() + "）";
//		excelLog.setGroupName(groupName);
//
//		JSONObject childRule = JSON.parseObject(userGroupEntity.getRules());
//		QueryWrapper<ZUserTagEntity> wrapperChild = calculateService.zUserTagParseRuleExport(childRule);
//		QueryWrapper<ZUserTagEntity> wrapperParent = Wrappers.query();
//		wrapperParent.ge("THEMONTH_V", params.getMonthStart()).le("THEMONTH_V", params.getMonthEnd());
//
//		if (excelLogService.save(excelLog)) {
//			excelService.excute(params, wrapperChild, excelLog);
//		} else {
//			return R.fail("导出失败");
//		}
//
//		return R.success("正在导出");
//	}

	/**
	 * 普通的导出
	 *
	 * @param params
	 * @return
	 */
	private R<Object> handleExport(UserProfileExcelLog excelLog, UserGroupEntity userGroupEntity,
			ExcelExportParam params) {
		if (excelLogService.save(excelLog)) {
//			JSONObject rule = JSON.parseObject(userGroupEntity.getRules());
			QueryWrapper<ZUserTagEntity> wrapper = userGroupRuleService.parse(userGroupEntity);
			excelService.excute(params, wrapper, excelLog);
		} else {
			return R.fail("导出失败");
		}

		return R.success("正在导出");
	}

	@ApiOperation("下载Excel")
	@GetMapping("/download/{id}")
	public void download(@PathVariable(value = "id", required = true) Long id, HttpServletResponse response)
			throws IOException {
		UserProfileExcelLog excelLog = excelLogService.getById(id);
		if (excelLog != null) {
			File file = new File(excelLog.getFilePath());
			if (file != null && file.exists() && file.isFile()) {
				response.setContentType("application/vnd.ms-excel");
				response.setCharacterEncoding("utf-8");
				// 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
				String fileName = URLEncoder.encode(excelLog.getGroupName(), "UTF-8");
				response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
				response.getOutputStream().write(FileUtil.readBytes(file));
				response.getOutputStream().flush();
			}
		}
	}

	@ApiOperation("分页查询")
	@RequestMapping(value = "/page", method = RequestMethod.POST)
	public ResultSegment page(@RequestBody ExcelExportPageParam excelParam) {
		Long sysUserId = getUser().getUserId();
		if (sysUserId == null) {
			return new ResultSegment(false, 400, "没有权限");
		}

		QueryWrapper<UserProfileExcelLog> wrapper = new QueryWrapper<>();
		wrapper.orderByDesc("id");
		wrapper.like(StrUtil.isNotEmpty(excelParam.getGroupName()), "group_name", excelParam.getGroupName());
		Page<UserProfileExcelLog> logPage = new Page<>(excelParam.getPageNo(), excelParam.getPageSize());
		Page<UserProfileExcelLog> excelLogPage = excelLogService.page(logPage, wrapper);
		ResultSegment resultSegment = new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG,
				excelLogPage);
		return resultSegment;
	}

	@ApiOperation("删除Excel任务")
	@GetMapping("/delete/{id}")
	public R<String> delete(@PathVariable(value = "id", required = true) Long id) throws IOException {
		Long sysUserId = getUser().getUserId();
		if (sysUserId == null) {
			return R.fail("没有权限");
		}

		UserProfileExcelLog log = excelLogService.getById(id);
		if (log == null) {
			return R.fail("任务不存在");
		}

		if (sysUserId.longValue() != log.getSysUserId().longValue()) {
			return R.fail("没有权限");
		}

		if (excelLogService.removeById(id)) {
			File file = new File(log.getFilePath());
			if (file != null && file.exists() && file.isFile()) {
				file.delete();
			}
		}
		return R.success("删除成功");
	}

	@ApiOperation("根据Cpid和月份导出手机号")
	@ApiImplicitParams({ @ApiImplicitParam(name = "cpid", value = "产品CPID，比如：kuaishou", required = true),
			@ApiImplicitParam(name = "month", value = "精度到月份的日期，示例：202106", required = true) })
	@GetMapping("/exportByCpid")
	public R<Kv> exportByCpid(String cpid, String month) {
		if (StringUtils.isBlank(cpid) || StringUtils.isBlank(month)) {
			return R.fail("参数异常");
		}

		cpidUserIdService.handle(cpid, month);
		String newUserPath = handleCpid(cpid, month, 0);
		log.info("export new user finish");
		String oldUserPath = handleCpid(cpid, month, 1);
		log.info("export old user finish");
		Kv ret = Kv.create();
		ret.set("newUser", newUserPath);
		ret.set("oldUser", oldUserPath);
		return R.data(ret);
	}

	private String handleCpid(String cpid, String month, int type) {
		List<List<String>> headList = new ArrayList<>();
		List<String> phoneHeader = new ArrayList<>();
		phoneHeader.add("手机号");
		headList.add(phoneHeader);

		String sheetName = cpid + "_" + month + (type == 0 ? "_新用户" : "_老用户");
		String tempFileName = sheetName + ".xlsx";
		String filePath = excelPath + "/" + tempFileName;
		ExcelWriter excelWriter = EasyExcel.write(filePath).head(headList).build();

		int page = 1;
		while (true) {
			List<MidCpidUserId> list = cpidUserIdService.getPhoneList(type, page);
			if (list == null || list.size() == 0) {
				// 没有数据
				break;
			}

			try {
				List<List<String>> dataList = new ArrayList<>();
				list.forEach(item -> {
					List<String> phoneList = new ArrayList<>();
					phoneList.add(item.getUserId());
					dataList.add(phoneList);
				});
				excelWriter.write(dataList, EasyExcel.writerSheet(sheetName).build());
				page += 1;
			} catch (Exception e) {
				log.info("exportByCpid error is " + e.getMessage());
				FileUtils.delete(new File(filePath));
			}

			if (list.size() < MidCpidUserIdService.PAGE_SIZE) {
				// 已查询完
				break;
			}
		}
		if (excelWriter != null) {
			excelWriter.finish();
		}
		return "/uploads/excel/" + tempFileName;
	}
}
