package org.springblade.useranalysis.controller;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springblade.useranalysis.common.utils.CommonUtil;
import org.springblade.useranalysis.common.utils.Constants;
import org.springblade.useranalysis.common.utils.ErrorEnum;
import org.springblade.useranalysis.dto.RequestPageParam;
import org.springblade.useranalysis.dto.ResultSegment;
import org.springblade.useranalysis.entity.UserGroupAnalysisRequest;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.entity.UserGroupRequest;
import org.springblade.useranalysis.service.UserGroupAnalysisService;
import org.springblade.useranalysis.service.UserGroupCalculateService;
import org.springblade.useranalysis.service.UserGroupService;
import org.springblade.useranalysis.service.UserProfileService;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;

/**
 * 用户人群分析
 */
@RestController
@AllArgsConstructor
@RequestMapping("/v1/UserGroup")
@Api(value = "mysql用户人群管理", tags = "用户人群管理")
public class UserGroupController {

	private final UserGroupAnalysisService userGroupAnalysisService;
	private final UserGroupCalculateService userGroupCalculateService;
	private final UserProfileService userProfileService;
	private final UserGroupService userGroupService;

	/**
	 * 新增用户人群
	 */
	@ApiOperation("新增用户人群")
	@ApiOperationSupport(order = 1)
	@RequestMapping(value = "/addUsergroup", method = RequestMethod.POST)
	public JSONObject addUsergroup(@RequestBody UserGroupEntity userGroupEntity) {
		if (userGroupService.save(userGroupEntity)) {
			return CommonUtil.successJson();
		} else {
			return CommonUtil.errorJson(ErrorEnum.E_500);
		}

	}

	/**
	 * 删除用户人群
	 */
	@ApiOperation("删除用户人群")
	@ApiOperationSupport(order = 2)
	@RequestMapping(value = "/deleteUsergroup", method = RequestMethod.POST)
	public JSONObject deleteUsergroup(@RequestParam(value = "groupId", required = false) String groupId) {
		LambdaQueryWrapper<UserGroupEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
		lambdaQueryWrapper.eq(UserGroupEntity::getGroupId, groupId);
		if (userGroupService.remove(lambdaQueryWrapper)) {
			return CommonUtil.successJson();
		} else {
			return CommonUtil.errorJson(ErrorEnum.E_10010);
		}

	}

	/**
	 * 修改用户人群
	 */
	@ApiOperation("修改用户人群")
	@ApiOperationSupport(order = 3)
	@RequestMapping(value = "/updateUsergroup", method = RequestMethod.POST)
	public JSONObject updateUsergroup(@RequestBody UserGroupEntity UserGroupEntity) {
		if (userGroupService.updateById(UserGroupEntity)) {
			return CommonUtil.successJson();
		} else {
			return CommonUtil.errorJson(ErrorEnum.E_10010);
		}

	}

	/**
	 * 用户人群列表 分页展示
	 */
	@ApiOperation("用户人群列表 分页展示")
	@ApiOperationSupport(order = 4)
	@RequestMapping(value = "/usergroupList", method = RequestMethod.POST)
	public ResultSegment usergroupList(@RequestBody RequestPageParam requestList) {
		JSONObject map = userGroupService.listUsergroup(requestList.getPageNo(), requestList.getPageSize());
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, map);
	}

	/**
	 * 根据人群名称进行查询
	 */

	@ApiOperation("根据人群名称进行查询")
	@ApiOperationSupport(order = 5)
	@RequestMapping(value = "/searchUsergroup", method = RequestMethod.POST)
	public ResultSegment searchUsergroup(@RequestParam(value = "group_name", required = false) String group_name) {
		JSONObject map = userGroupService.searchUsergroup(group_name);
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, map);
	}

	/**
	 * 根据人群ID进行查询
	 */
	@ApiOperation("根据人群ID进行查询")
	@ApiOperationSupport(order = 6)
	@RequestMapping(value = "/searchUsergroupById", method = RequestMethod.POST)
	public ResultSegment searchUsergroupById(@RequestParam(value = "group_id", required = false) String group_id) {
		LambdaQueryWrapper<UserGroupEntity> objectLambdaQueryWrapper = new LambdaQueryWrapper<>();
		List<UserGroupEntity> list = userGroupService
				.list(objectLambdaQueryWrapper.eq(UserGroupEntity::getGroupId, group_id));
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, list);
	}

	/**
	 * 用户画像 —— 圈人计算 bingo
	 */
	@ApiOperation("用户画像——圈人计算")
	@ApiOperationSupport(order = 7)
	@RequestMapping(value = "/usergroupCalculate", method = RequestMethod.POST)
	public ResultSegment usergroupCalculate(@RequestBody UserGroupRequest userGroupRequest) {
		JSONObject userGroupCount = userGroupCalculateService.usergroupCalculate(userGroupRequest);
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, userGroupCount);
	}

	/**
	 * 用户画像 —— 圈人计算，支持以导入的人群为基础
	 */
	@ApiOperation("用户画像——圈人计算(支持以导入的人群为基础)")
	@ApiOperationSupport(order = 7)
	@RequestMapping(value = "/usergroupCalculateBaseImport", method = RequestMethod.POST)
	public ResultSegment usergroupCalculateBaseImport(@RequestBody UserGroupRequest userGroupRequest) {
		boolean isBaseImport = StringUtils.isNotBlank(userGroupRequest.getImportId());
		JSONObject userGroupCount = null;
		if (isBaseImport) {
			userGroupCount = userGroupCalculateService.usergroupCalculateBaseImport(userGroupRequest);
		} else {
			userGroupCount = userGroupCalculateService.usergroupCalculate(userGroupRequest);
		}
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, userGroupCount);
	}

	/**
	 * 用户画像 —— 人群多维分析-选择人群
	 */
	@RequestMapping(value = "/distinctGroupname", method = RequestMethod.POST)
	@ApiOperationSupport(order = 8)
	@ApiOperation("人群多维分析-选择人群")
	public ResultSegment distinctGroupname() {
		List<JSONObject> groupName = userProfileService.distinctGroupname();
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, groupName);
	}

	/**
	 * 用户画像 —— 人群多维分析 ok
	 */

	@ApiOperation("用户画像——人群多维分析")
	@ApiOperationSupport(order = 9)
	@RequestMapping(value = "/usergroupAnalysis", method = RequestMethod.POST)
	public ResultSegment usergroupAnalysis(@RequestBody UserGroupAnalysisRequest userGroupAnalysisRequest) {
		JSONObject usergroupInfo = userGroupAnalysisService.usergroupCalculate(userGroupAnalysisRequest);
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, usergroupInfo);
	}

	/**
	 * 用户画像 —— 人群多维分析 —— 下拉菜单 筛选分析维度
	 */
	@ApiOperation("用户画像 —— 人群多维分析 —— 下拉菜单 筛选分析维度")
	@ApiOperationSupport(order = 10)
	@RequestMapping(value = "/usergroupAnalysisList", method = RequestMethod.POST)
	public JSONObject usergroupAnalysisList() {
		return userProfileService.usergroupAnalysisList();
	}

	/**
	 * 用户画像 —— 人群多维分析 —— 填写标签权重值
	 */

	@ApiOperation("用户画像 —— 人群多维分析 —— 填写标签权重值")
	@ApiOperationSupport(order = 11)
	@RequestMapping(value = "/usergroupAnalysisTags", method = RequestMethod.POST)
	public JSONObject usergroupAnalysisTags(@RequestParam(value = "level_2_name", required = false) String level_2_name,
			@RequestParam(value = "tagType", required = false) String tag_type) {
		return userProfileService.usergroupAnalysisTags(level_2_name, tag_type);
	}

	/**
	 * 选择人群分析维度
	 */

	@ApiOperation("选择人群分析维度")
	@ApiOperationSupport(order = 12)
	@RequestMapping(value = "/analysisDimension", method = RequestMethod.POST)
	public ResultSegment analysisDimension() {
		List<JSONObject> map = userGroupService.analysisDimension();
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, map);
	}

	/**
	 * 选择人群分析维度-备用 分布分析模块用
	 */
	@ApiOperation("选择人群分析维度-备用 分布分析模块用")
	@ApiOperationSupport(order = 13)
	@RequestMapping(value = "/dimensionDistribute", method = RequestMethod.POST)
	public ResultSegment dimensionDistribute() {
		List<JSONObject> map = userGroupService.dimensionDistribute();
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, map);
	}

}
