/**
 *
 */
package org.springblade.useranalysis.controller;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import org.apache.commons.codec.digest.DigestUtils;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.useranalysis.common.utils.EnDecHelper;
import org.springblade.useranalysis.dto.UserGroupAnalysisRequest;
import org.springblade.useranalysis.dto.UserGroupAnalysisResult;
import org.springblade.useranalysis.entity.UserGroupAsyncDataLog;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.entity.clickhouse.ZUserTagEntity;
import org.springblade.useranalysis.enums.UserGroupProcessStatus;
import org.springblade.useranalysis.enums.UserGroupUseStatus;
import org.springblade.useranalysis.service.UserGroupService;
import org.springblade.useranalysis.service.normal.UserGroupAsyncDataLogService;
import org.springblade.useranalysis.service.normal.UserGroupRuleService;
import org.springblade.useranalysis.service.normal.UserGroupSumDataManager;
import org.springblade.useranalysis.service.normal.UserGroupSumDataService;
import org.springblade.useranalysis.service.normal.UserGroupUserIdService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.http.HttpUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
//@AllArgsConstructor
@RequestMapping("/v2/userGroup")
@Api(value = "[新版]用户人群管理", tags = "[新版]用户人群管理")
public class UserGroupNewController extends BladeController {

	@Autowired
	private UserGroupAsyncDataLogService userGroupAsyncDataLogService;

	@Autowired
	private UserGroupService userGroupService;

	@Autowired
	private UserGroupSumDataService userGroupSumDataService;

	@Autowired
	private UserGroupUserIdService userGroupUserIdService;

	@Autowired
	private UserGroupSumDataManager userGroupSumDataManager;

	@Autowired
	private UserGroupRuleService userGroupRuleService;

	@Value("${smartOrder.insertUrl}")
	private String insertUrl;

	@Value("${smartOrder.updateUrl}")
	private String updateUrl;

	@Value("${smartOrder.deleteUrl}")
	private String deleteUrl;

	@Value("${smartOrder.md5pwd}")
	private String md5pwd;

	@Value("${smartOrder.pushUseridUrl}")
	private String pushUseridUrl;

	@PostMapping("/testGet")
	@ApiOperation(value = "【测试】查询人群", notes = "返回计算的人群数量")
	@ApiOperationSupport(order = 0)
	public R<Long> testCount(String userGroupId){
		QueryWrapper<ZUserTagEntity> queryWrapper = userGroupRuleService.parse(userGroupId);
		return R.data(userGroupUserIdService.testDataCount(queryWrapper));
	}

	/**
	 * 用户人群列表 分页展示
	 */
	@ApiOperation("用户人群列表分页展示")
	@ApiOperationSupport(order = 0)
	@GetMapping(value = "/list")
	public R<IPage<UserGroupEntity>> usergroupList(UserGroupEntity userGroupEntity, Query query) {
		QueryWrapper<UserGroupEntity> queryWrapper = Condition.getQueryWrapper(userGroupEntity);
		return R.data(userGroupService.page(Condition.getPage(query), queryWrapper.lambda().orderByDesc(UserGroupEntity::getUpdateTime)));
	}

	/**
	 * 新增用户人群
	 */
	@ApiOperation("新增用户人群")
	@ApiOperationSupport(order = 1)
	@RequestMapping(value = "/add", method = RequestMethod.POST)
	public R<String> addUsergroup(@RequestBody UserGroupEntity userGroupEntity) {
		userGroupEntity.setReachCount("0");
		userGroupEntity.setRuleVersion(1); //设置为新版本rule规则
		userGroupEntity.setCreater(getUser().getNickName());
		userGroupEntity.setProcessStatus(UserGroupProcessStatus.waiting);
		userGroupEntity.setCreateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
		userGroupEntity.setUpdateTime(new Date());
		List<DateTime> rangeToList = DateUtil.rangeToList(userGroupEntity.getStartAnalyseDate(), userGroupEntity.getEndAnalyseDate(), DateField.MONTH);
		List<String> analyseDateList = rangeToList.stream().map(item -> DateUtil.format(item, "yyyyMM")).sorted().collect(Collectors.toList());
		userGroupEntity.setAnalyseDateJson(JSON.toJSONString(analyseDateList));
		if (userGroupEntity.getGroupUse() == 1){//智能订购页
			userGroupEntity.setUseStatus(UserGroupUseStatus.waiting);
		}
		if (userGroupService.save(userGroupEntity)) {
//			if (userGroupEntity.getGroupUse() == 1){
//				//处理智能订购页逻辑,组装参数发送http请求，判断返回结果
//				Map<String,Object> paraMap = new HashMap();
//				long timestamp=System.currentTimeMillis()/1000;
//				paraMap.put("timestamp", timestamp+"");
//				paraMap.put("signature", DigestUtils.md5Hex(timestamp+md5pwd));
//				//组装dataArray
//				JSONArray dataArray= new JSONArray();
//				JSONObject data = new JSONObject();
//				data.put("name",userGroupEntity.getGroupName());
//				data.put("tagId",userGroupEntity.getGroupId()+"");
//				dataArray.add(data);
//				paraMap.put("data",dataArray);
//				String params = GsonUtils.toJson(paraMap);
//				String results = HttpUtils.doPostByJson(insertUrl, null, params);
//				JSONObject jsonObject =JSONObject.parseObject(results);
//				if (jsonObject != null) {
//					String code = jsonObject.getString("code");
//					if ("10000".equals(code)){
//						//更新userGroupUseStatus.processing
////					userGroupEntity.setUseStatus(UserGroupUseStatus.processing);
////					userGroupService.updateById(userGroupEntity);
//						log.info("【更新智能订购分群】已更新智能订购分群｜groupid={}", userGroupEntity.getGroupId());
//					}
//				}
//			}
			//开始数据预处理
			userGroupSumDataManager.execute(userGroupEntity.getGroupId());
			return R.success("添加成功，数据预处理已进入队列...");
		} else {
			return R.fail("添加失败");
		}
	}

	/**
	 * 删除用户人群
	 */
	@ApiOperation("删除用户人群")
	@ApiOperationSupport(order = 2)
	@RequestMapping(value = "/delete", method = RequestMethod.POST)
	public R<String> deleteUsergroup(@RequestParam(value = "groupId", required = true) String groupId) {
		UserGroupEntity userGroupEntity = userGroupService.getById(groupId);
		if (userGroupService.removeById(groupId)) {
			//删除预处理表数据
			userGroupUserIdService.deleteByGroupId(groupId);
			userGroupSumDataService.deleteByGroupId(groupId);
			log.info("【删除人群】已删除预处理表数据｜groupid={}", groupId);

			//删除历史处理结果
			userGroupAsyncDataLogService.remove(Wrappers.<UserGroupAsyncDataLog>lambdaQuery().eq(UserGroupAsyncDataLog::getGroupId, groupId));
			log.info("【删除人群】已删除历史处理结果数据｜groupid={}", groupId);
			if (userGroupEntity.getGroupUse() == 1){
				//处理智能订购页逻辑,组装参数发送http请求，判断返回结果
				Map<String,Object> paraMap = new HashMap();
				long timestamp=System.currentTimeMillis()/1000;
				paraMap.put("timestamp", timestamp+"");
				paraMap.put("signature", DigestUtils.md5Hex(timestamp+md5pwd));
				paraMap.put("tagId",groupId);
//				String params = GsonUtils.toJson(paraMap);
//				String results = HttpUtils.doPostByJson(deleteUrl, null, params);
				String params = JSON.toJSONString(paraMap);
				String results = HttpUtil.post(deleteUrl, params);
				JSONObject jsonObject =JSONObject.parseObject(results);
				if (jsonObject != null) {
					String code = jsonObject.getString("code");
					if ("10000".equals(code)){
						log.info("【删除智能订购分群】已删除智能订购分群｜groupid={}", groupId);
					}
				}
			}
			return R.success("删除成功");
		} else {
			return R.fail("删除失败");
		}
	}

	/**
	 *
	 * @param userGroupEntity
	 * @return
	 */
	@ApiOperation(value = "修改用户人群", notes = "该操作会删除预处理数据，重新开始跑数据")
	@ApiOperationSupport(order = 3)
	@RequestMapping(value = "/update", method = RequestMethod.POST)
	public R<String> updateUsergroup(@RequestBody UserGroupEntity userGroupEntity) {
		//重置数据处理状态
		userGroupEntity.setProcessStatus(UserGroupProcessStatus.waiting);
		userGroupEntity.setReachCount("0");
		userGroupEntity.setRuleVersion(1); //设置为新版本rule规则
		userGroupEntity.setUpdateTime(new Date());
		List<DateTime> rangeToList = DateUtil.rangeToList(userGroupEntity.getStartAnalyseDate(), userGroupEntity.getEndAnalyseDate(), DateField.MONTH);
		List<String> analyseDateList = rangeToList.stream().map(item -> DateUtil.format(item, "yyyyMM")).sorted().collect(Collectors.toList());
		userGroupEntity.setAnalyseDateJson(JSON.toJSONString(analyseDateList));
		if (userGroupService.updateById(userGroupEntity)) {
			if (userGroupEntity.getGroupUse() == 1){
				//处理智能订购页逻辑,组装参数发送http请求，判断返回结果
				Map<String,Object> paraMap = new HashMap();
				long timestamp=System.currentTimeMillis()/1000;
				paraMap.put("timestamp", timestamp+"");
				paraMap.put("signature", DigestUtils.md5Hex(timestamp+md5pwd));
				paraMap.put("name",userGroupEntity.getGroupName());
				paraMap.put("tagId",userGroupEntity.getGroupId()+"");
//				String params = GsonUtils.toJson(paraMap);
//				String results = HttpUtils.doPostByJson(updateUrl, null, params);
				String params = JSON.toJSONString(paraMap);
				String results = HttpUtil.post(updateUrl, params);
				JSONObject jsonObject =JSONObject.parseObject(results);
				if (jsonObject != null) {
					String code = jsonObject.getString("code");
					if ("10000".equals(code)){
						log.info("【更新智能订购分群】已更新智能订购分群｜groupid={}", userGroupEntity.getGroupId());
					}
				}
			}
			//删除预处理表数据
			userGroupUserIdService.deleteByGroupId(userGroupEntity.getGroupId());
			userGroupSumDataService.deleteByGroupId(userGroupEntity.getGroupId());
			log.info("【更新】已删除预处理表数据｜groupid={}", userGroupEntity.getGroupId());

			//删除历史处理结果
			userGroupAsyncDataLogService.remove(Wrappers.<UserGroupAsyncDataLog>lambdaQuery().eq(UserGroupAsyncDataLog::getGroupId, userGroupEntity.getGroupId()));
			log.info("【更新】已删除历史处理结果数据｜groupid={}", userGroupEntity.getGroupId());

			//再添加到数据预处理中
			userGroupSumDataManager.execute(userGroupEntity.getGroupId());
			log.info("【更新】重新添加到预处理队列｜groupid={}", userGroupEntity.getGroupId());
			return R.success("修改成功，数据正在重新生成...");
		} else {
			return R.fail("修改失败");
		}
	}

	/**
	 *
	 * @return
	 */
	@ApiOperation("重新跑数据")
	@ApiOperationSupport(order = 4)
	@PostMapping(value = "/reExecute")
	public R<String> reExecute(@RequestParam(value = "groupId", required = true) String groupId){
		//如果人群正在处理中或者队列中，不能处理
		UserGroupEntity groupEntity = userGroupService.getById(groupId);
		if(groupEntity.getProcessStatus() == UserGroupProcessStatus.processing) {
			return R.fail("该人群正在处理中，无需重跑...");
		}
		if(groupEntity.getProcessStatus() == UserGroupProcessStatus.queueing) {
			return R.fail("该人群正在队列中，无需重跑...");
		}
		UserGroupEntity userGroupEntity = new UserGroupEntity();
		userGroupEntity.setGroupId(groupId);
		userGroupEntity.setUpdateTime(new Date());
		userGroupService.updateById(userGroupEntity);
		//删除预处理表数据
		userGroupSumDataService.deleteByGroupId(userGroupEntity.getGroupId());
		log.info("【重新】已删除预处理表数据｜groupid={}", userGroupEntity.getGroupId());
		//删除历史处理结果
		userGroupAsyncDataLogService.remove(Wrappers.<UserGroupAsyncDataLog>lambdaQuery().eq(UserGroupAsyncDataLog::getGroupId, userGroupEntity.getGroupId()));
		log.info("【重新】已删除历史处理结果数据｜groupid={}", userGroupEntity.getGroupId());
		boolean addRet = userGroupSumDataManager.execute(groupId);
		return addRet ? R.success("用户人群进入队列，请稍后查询结果...") : R.success("用户人群已经在队列里，不用重复执行...");
	}

	/**
	 *
	 * @return
	 */
	@ApiOperation("重置标签分析时间")
	@ApiOperationSupport(order = 5)
	@PostMapping(value = "/restAnalyseDate")
	public R<String> restAnalyseDate(@RequestBody UserGroupEntity userGroupEntity){
		List<DateTime> rangeToList = DateUtil.rangeToList(userGroupEntity.getStartAnalyseDate(), userGroupEntity.getEndAnalyseDate(), DateField.MONTH);
		List<String> analyseDateList = rangeToList.stream().map(item -> DateUtil.format(item, "yyyyMM")).sorted().collect(Collectors.toList());
		userGroupEntity.setAnalyseDateJson(JSON.toJSONString(analyseDateList));
		userGroupEntity.setUpdateTime(new Date());
		userGroupService.updateById(userGroupEntity);
		//删除预处理结果
		userGroupSumDataService.deleteByGroupId(userGroupEntity.getGroupId());
		log.info("【重置标签分析时间】已删除预处理表数据｜groupid={}", userGroupEntity.getGroupId());
		//删除历史处理结果
		userGroupAsyncDataLogService.remove(Wrappers.<UserGroupAsyncDataLog>lambdaQuery().eq(UserGroupAsyncDataLog::getGroupId, userGroupEntity.getGroupId()));
		log.info("【重置标签分析时间】已删除历史处理结果数据｜groupid={}", userGroupEntity.getGroupId());
		//开始进入重跑队列
		boolean addRet = userGroupSumDataManager.execute(userGroupEntity.getGroupId());
		return addRet ? R.success("用户人群进入队列，请稍后查询结果...") : R.success("用户人群已经在队列里，不用重复执行...");
	}

	/**
	 *
	 * @param groupId
	 * @return
	 */
	@ApiOperation("查询跑数据的历史记录")
	@ApiOperationSupport(order = 6)
	@GetMapping("/dataLogList")
	public R<List<UserGroupAsyncDataLog>> dataLog(@RequestParam(value = "groupId", required = true) String groupId){
		return R.data(userGroupAsyncDataLogService.list(Wrappers.<UserGroupAsyncDataLog>lambdaQuery().eq(UserGroupAsyncDataLog::getGroupId, groupId)));
	}

	/**
	 *
	 * @param groupId
	 * @param level2Id
	 * @return
	 */
	@ApiOperation("人群多维分析")
	@ApiOperationSupport(order = 7)
	@PostMapping("/analysis")
	public R<List<UserGroupAnalysisResult>> analysis(@RequestBody UserGroupAnalysisRequest userGroupAnalysisRequest){
		return R.data(userGroupSumDataService.analyis(userGroupAnalysisRequest));
	}


	/**
	 * 重推智能订购页数据
	 */
	@ApiOperation("重推智能订购页数据")
	@ApiOperationSupport(order = 8)
	@RequestMapping(value = "/rePushUserId", method = RequestMethod.POST)
	public R<String> rePushUserId(@RequestParam(value = "groupId", required = true) String groupId) {
		UserGroupEntity groupEntity = userGroupService.getById(groupId);
		if (groupEntity != null){
			try{
				//先删除，后添加
				//处理智能订购页逻辑,组装参数发送http请求，判断返回结果
				Map<String,Object> paraMap = new HashMap();
				long timestamp=System.currentTimeMillis()/1000;
				paraMap.put("timestamp", timestamp+"");
				paraMap.put("signature", DigestUtils.md5Hex(timestamp+md5pwd));
				paraMap.put("tagId",groupId);
//				String params = GsonUtils.toJson(paraMap);
//				String results = HttpUtils.doPostByJson(deleteUrl, null, params);
				String params = JSON.toJSONString(paraMap);
				String results = HttpUtil.post(deleteUrl, params);
				JSONObject jsonObject =JSONObject.parseObject(results);
				if (jsonObject != null) {
					String code = jsonObject.getString("code");
					if ("10000".equals(code)){
						Map<String,Object> map = new HashMap();
						long timestamp1=System.currentTimeMillis()/1000;
						map.put("timestamp", timestamp1+"");
						map.put("signature", DigestUtils.md5Hex(timestamp1+md5pwd));
						//组装dataArray
						JSONArray dataArray= new JSONArray();
						JSONObject data = new JSONObject();
						data.put("name",groupEntity.getGroupName());
						data.put("tagId",groupEntity.getGroupId()+"");
						data.put("num",groupEntity.getReachCount());
						dataArray.add(data);
						map.put("data",dataArray);
//						String insertParam = GsonUtils.toJson(map);
//						String insertResult = HttpUtils.doPostByJson(insertUrl, null, insertParam);
						String insertParam = JSON.toJSONString(map);
						String insertResult = HttpUtil.post(insertUrl, params);
						JSONObject insertObject =JSONObject.parseObject(insertResult);
						if (insertObject != null) {
							String objectString = insertObject.getString("code");
							if ("10000".equals(objectString)){
								//更新userGroupUseStatus.processing
								groupEntity.setUseStatus(UserGroupUseStatus.processing);
								userGroupService.updateById(groupEntity);
								log.info("【重推智能订购页数据---添加智能订购分群】已添加智能订购分群｜groupid={}", groupId);
								Integer limit=50000;
								String userId="0";
								while (true){
									List<String> resultList = userGroupUserIdService.selectByGroupId(groupId,userId,limit);
									if (resultList == null || resultList.isEmpty()){
										break;
									}
									//处理数据
									pushData(groupId,resultList);
									userId = resultList.get(resultList.size() - 1);
								}
								groupEntity.setUseStatus(UserGroupUseStatus.success);
								userGroupService.updateById(groupEntity);

							}
						}
					}
				}
			}catch (Exception e){
				log.error(ExceptionUtil.stacktraceToString(e));
				groupEntity.setUseStatus(UserGroupUseStatus.failure);
				userGroupService.updateById(groupEntity);
				return R.fail("重推智能订购页数据失败！");
			}
		}else {
			return R.fail("不存在该分群！");
		}
		return R.success("重推智能订购页数据成功！");
	}



	/**
	 * 同步智能订购页数据
	 */
	@ApiOperation("同步智能订购页数据")
	@ApiOperationSupport(order = 9)
	@RequestMapping(value = "/updateUseStatus", method = RequestMethod.POST)
	public R<String> updateUseStatus(@RequestParam(value = "groupId", required = true) String groupId) {
		UserGroupEntity userGroupEntity = userGroupService.getById(groupId);
		if (userGroupEntity.getGroupUse() == 1){
			//处理智能订购页逻辑,组装参数发送http请求，判断返回结果
			Map<String,Object> paraMap = new HashMap();
			long timestamp=System.currentTimeMillis()/1000;
			paraMap.put("timestamp", timestamp+"");
			paraMap.put("signature", DigestUtils.md5Hex(timestamp+md5pwd));
			//组装dataArray
			JSONArray dataArray= new JSONArray();
			JSONObject data = new JSONObject();
			data.put("name",userGroupEntity.getGroupName());
			data.put("tagId",userGroupEntity.getGroupId()+"");
			data.put("num",userGroupEntity.getReachCount());
			dataArray.add(data);
			paraMap.put("data",dataArray);
//			String params = GsonUtils.toJson(paraMap);
//			String results = HttpUtils.doPostByJson(insertUrl, null, params);
			String params = JSON.toJSONString(paraMap);
			String results = HttpUtil.post(insertUrl, params);
			JSONObject jsonObject =JSONObject.parseObject(results);
			if (jsonObject != null) {
				String code = jsonObject.getString("code");
				if ("10000".equals(code)){
					//更新userGroupUseStatus.processing
					userGroupEntity.setUseStatus(UserGroupUseStatus.processing);
					userGroupService.updateById(userGroupEntity);
					log.info("【同步智能订购页数据】已添加智能订购分群｜groupid={}", groupId);
					return R.success("同步智能订购页数据成功！");
				}
			}
		}
		return R.fail("同步智能订购页数据失败！");
	}


	/**
	 * 处理流程
	 * @param groupId
	 * @param resultList
	 * @throws Exception
	 */
	public void pushData(String groupId,List<String>resultList) throws Exception {
		if (resultList != null && resultList.size() >0 ){
			String userIds= resultList.parallelStream().map(EnDecHelper::encode).filter(Objects::nonNull).collect(Collectors.joining(","));
			Map<String,Object> paraMap = new HashMap();
			long timestamp=System.currentTimeMillis()/1000;
			paraMap.put("timestamp", timestamp+"");
			paraMap.put("signature", DigestUtils.md5Hex(timestamp+md5pwd));
			paraMap.put("userId", userIds);
//			String params = GsonUtils.toJson(paraMap);
//			String results=HttpUtils.doPostByJson(pushUseridUrl+groupId, null, params);
			String params = JSON.toJSONString(paraMap);
			String results=HttpUtil.post(pushUseridUrl+groupId, params);
			log.info("【重推结果】重推结果｜results={}", results);
			JSONObject jsonObject =JSONObject.parseObject(results);
			if (jsonObject != null) {
				String code = jsonObject.getString("code");
				if (!"10000".equals(code)){
					throw new RuntimeException("重推失败!");
				}
			}

		}
	}
}
