package org.springblade.useranalysis.mapper.clickhouse;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springblade.useranalysis.entity.clickhouse.UserEvent;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;

/**
 * (UserEvent)表数据库访问层
 *
 * @author limeng
 * @since 2021-07-06 14:38:03
 */
public interface UserEventMapper extends BaseMapper<UserEvent> {

	@Select({
		"select toDate('${startDate}') as ds, ${selectColmns}",
		"from (",
		" with",
		"	d_date as dt,",
		"  	toDate('${startDate}') as firstDate",
		" SELECT",
		"   anonymous_id,", 
		"   retention(",
		"     ${retentionColmns}",
		"   ) AS r",
		"   from ads.user_event",
		"   ${ew.customSqlSegment}",
		"	group by anonymous_id",
		")"
	})
    List<Map<String, Object>> remainAnalysisData(@Param("startDate") String startDate,
                                                 @Param("selectColmns") String selectColmns,
                                                 @Param("retentionColmns") String retentionColmns,
                                                 @Param(Constants.WRAPPER) QueryWrapper<UserEvent> TagWrapper);

    @Select("select ${targets} dt, level_index, count(1) as count from" +
            "(" +
            "    select ${targets} dt, user_id, " +
            "        arrayWithConstant(level, 1) levels, " +
            "        arrayJoin(arrayEnumerate(levels)) level_index" +
            "      from (" +
            "		with " +
            "		  ${selectField},"+
            "		  d_timestamp as ds"+
            "        select " +
            "        ${targets} dt, anonymous_id as user_id," +
            "          windowFunnel(${windowSeconds})(" +
            "            ds  ${eventList} " +
            "          ) AS level" +
            "        from ads.user_event " +
            "        ${ew.customSqlSegment} " +
            "        group by ${targets} dt, user_id" +
            "    )" +
            ")" +
            "group by ${targets} dt, level_index " +
            "order by ${targets} dt, level_index")
    List<JSONObject> hopperAnalysis(@Param("targets") String targets,
                                    @Param("eventList") String eventList,
                                    @Param("windowSeconds") Long windowSeconds,
                                    @Param("selectField") String selectField,
                                    @Param(Constants.WRAPPER) QueryWrapper<UserEvent> TagWrapper);

    @Select({
    	"<script>",
		"select ${target} ${countSql} from(",
		"with d_date as dt",
		"select ${target} anonymous_id, count(1) as cnt",
		"from ads.user_event",
		"${ew.customSqlSegment}",
		"group by ${target} anonymous_id",
		") ",
		"<if test='targetWithoutComma != null and targetWithoutComma != \"\"'>group by ${targetWithoutComma}</if>",
		"</script>"
    })
	List<JSONObject> distributeAnalysis(@Param("target") String target, 
										@Param("targetWithoutComma") String targetWithoutComma,
										@Param("countSql") String countSql,
										@Param(Constants.WRAPPER) QueryWrapper<UserEvent> queryWrapper);
    
    @Select({
    	"<script>",
    	"select ${target} ${countSql} from(",
    	"	with toHour(d_timestamp) as hours, d_date as dt",
    	" select ${target} hours, uniqCombined(anonymous_id) as num",
    	" from ads.user_event",
    	" ${ew.customSqlSegment}",
    	" group by ${target} hours",
    	") ",
		"<if test='targetWithoutComma != null and targetWithoutComma != \"\"'>group by ${targetWithoutComma}</if>",
		"</script>"
    })
    List<JSONObject> distributeAnalysisOfTime(@Param("target") String target, 
											@Param("targetWithoutComma") String targetWithoutComma,
											@Param("countSql") String countSql,
											@Param(Constants.WRAPPER) QueryWrapper<UserEvent> queryWrapper);
    
    @Select({
    	"<script>",
		"select ${target} ${countSql} from(",
		"with d_date as dt",
		"select ${target} uniqCombined(${attrCode}) as cnt",
		"from ads.user_event",
		"${ew.customSqlSegment}",
		"<if test='targetWithoutComma != null and targetWithoutComma != \"\"'>group by ${targetWithoutComma}</if>",
		") ",
		"<if test='targetWithoutComma != null and targetWithoutComma != \"\"'>group by ${targetWithoutComma}</if>",
		"</script>"
    })
    List<JSONObject> distributeAnalysisOfUniqAttr(
    							@Param("attrCode") String attrCode,
    							@Param("target") String target, 
								@Param("targetWithoutComma") String targetWithoutComma,
								@Param("countSql") String countSql,
								@Param(Constants.WRAPPER) QueryWrapper<UserEvent> queryWrapper);
    
    @Select("select aid from ads.user_event where ptype = #{value} and aid is not null group by aid")
    List<String> findAidList(String ptype);
//	@SqlParser(filter=true)
//    @Select("select a.${eventField} as eventField , a.Acount, b.Bcount, c.Ccount , d.Dcount ,  e.Ecount , f.Fcount  from (select ${eventField}  , count(1) as Acount from ( select distinct_id ,${eventField},count(1)  as count from user_event    ${ew.customSqlSegment} group by distinct_id,${eventField})   where count>=1 and  count<3 group by  ${eventField} ) a" +
//            "LEFT JOIN " +
//            "( select ${eventField} , count(1)  as Bcount  from ( select distinct_id ,${eventField},count(1)  as count from user_event    ${ew.customSqlSegment} group by distinct_id,${eventField})   where count>=3 and  count<5 group by  ${eventField} ) b on a.${eventField} =b.${eventField}" +
//            "LEFT JOIN " +
//            "( select ${eventField} , count(1) as Ccount from ( select distinct_id ,${eventField},count(1)  as count from user_event    ${ew.customSqlSegment} group by distinct_id,${eventField})   where count>=5 and  count<10 group by  ${eventField} ) c  on a.${eventField} =c.${eventField}" +
//            "LEFT JOIN " +
//            "( select ${eventField} , count(1) as Dcount from ( select distinct_id ,${eventField},count(1)  as count from user_event    ${ew.customSqlSegment} group by distinct_id,${eventField})   where count>=10 and  count<20 group by  ${eventField}  )d  on a.${eventField} =d.${eventField}" +
//            "LEFT JOIN " +
//            "( select ${eventField} , count(1) as Ecount from ( select distinct_id ,${eventField},count(1)  as count from user_event    ${ew.customSqlSegment} group by distinct_id,${eventField})   where count>=20 and count<50   group by  ${eventField}  ) e  on a.${eventField} =e.${eventField}" +
//            "LEFT JOIN " +
//            "( select ${eventField} , count(1) as Fcount from ( select distinct_id ,${eventField},count(1)  as count from user_event    ${ew.customSqlSegment} group by distinct_id,${eventField})   where count>=50   group by  ${eventField}  ) f  on a.${eventField} =f.${eventField}")
//    List<JSONObject> findCound(@Param("eventField") String eventField,
//                               @Param(Constants.WRAPPER) QueryWrapper<UserEvent> TagWrapper);
//	
//	@SqlParser(filter=true)
//	@Select("select count(1)  from(select distinct_id ,count(1) as count from user_event  ${ew.customSqlSegment} group by distinct_id) where count >= 1 and count<3" +
//		"UNION  all" +
//		"select count(1)  from(select distinct_id ,count(1) as count from user_event  ${ew.customSqlSegment} group by distinct_id) where count >= 3 and count<5" +
//		"UNION  all" +
//		"select count(1)  from(select distinct_id ,count(1) as count from user_event  ${ew.customSqlSegment} group by distinct_id) where count >= 5 and count<10" +
//		"UNION  all" +
//		"select count(1)  from(select distinct_id ,count(1) as count from user_event  ${ew.customSqlSegment} group by distinct_id) where count >= 10 and count<20" +
//		"UNION  all" +
//		"select count(1)  from(select distinct_id ,count(1) as count from user_event  ${ew.customSqlSegment} group by distinct_id) where count >= 20 and count<50" +
//		"UNION  all" +
//		"select count(1)  from(select distinct_id ,count(1) as count from user_event  ${ew.customSqlSegment} group by distinct_id) where count >= 50")
//	List<JSONObject> findCound0(@Param(Constants.WRAPPER) QueryWrapper<UserEvent> TagWrapper);
	
}
