package org.springblade.useranalysis.service.impl;

import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springblade.useranalysis.entity.TagColumn;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.entity.UserGroupRequest;
import org.springblade.useranalysis.entity.clickhouse.ZUserTagEntity;
import org.springblade.useranalysis.mapper.UserGroupMapper;
import org.springblade.useranalysis.mapper.clickhouse.UserTagsMapper;
import org.springblade.useranalysis.service.UserGroupCalculateService;
import org.springblade.useranalysis.service.UserProfileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import lombok.extern.slf4j.Slf4j;

/**
 * 用户人群计算模型
 */
@Slf4j
@Service
@DS("clickhouse")
public class UserGroupCalculateServiceImpl extends ServiceImpl<UserGroupMapper, UserGroupEntity>
		implements UserGroupCalculateService {

	@Autowired
	private UserProfileService userProfileService;

	@Autowired
	private UserTagsMapper userTagsMapper;
	
	// 解析人群规则
	public Integer parseRule(String ruleJson, String isSave) {
		// 拼接where 条件
		QueryWrapper<ZUserTagEntity> queryWrapper = zUserTagParseRule(JSONObject.parseObject(ruleJson));
		// int rtSize = userTagsMapper.selectCount(queryWrapper);
		int rtSize = userTagsMapper.userGroupCount(queryWrapper);
		return rtSize;
	}

	// 解析人群规则
	public Integer parseRuleBaseImport(String importId, String ruleJson, String isSave) {
		// 拼接where 条件
		QueryWrapper<ZUserTagEntity> queryWrapper = zUserTagParseRule(JSONObject.parseObject(ruleJson));
		// int rtSize = userTagsMapper.selectCount(queryWrapper);
		int rtSize = userTagsMapper.userGroupCountBaseImport(importId, queryWrapper);
		return rtSize;
	}

	/**
	 * 根据规则查询人群条件（用于导出excel，from william）
	 * 
	 * @param ruleJson
	 * @return
	 */
	public QueryWrapper<ZUserTagEntity> zUserTagParseRuleExport(JSONObject ruleJson) {
		JSONArray ruleColums = ruleJson.getJSONArray("ruledetails");
		String relationship = ruleJson.getString("relationship");

		QueryWrapper<ZUserTagEntity> queryWrapper = Wrappers.query();
		if (ruleColums.size() > 0) {
			for (int i = 0; i < ruleColums.size(); i++) {
				String key = ruleColums.getJSONObject(i).getString("key");
				String value = ruleColums.getJSONObject(i).getString("value");
				JSONArray jsonArr = JSONArray.parseArray(value);
				if (jsonArr.size() > 0) {
					queryWrapper = getQueryWrapper(relationship, (String) jsonArr.get(0), key, jsonArr, queryWrapper);
				}
			}
		}
		// 判断是否有日期
		JSONArray dateRangeArr = ruleJson.getJSONArray("daterange");
		if (dateRangeArr != null && dateRangeArr.size() == 2) {
			queryWrapper.ge("THEMONTH_V", dateRangeArr.getString(0).replace("-", "")).le("THEMONTH_V",
					dateRangeArr.getString(1).replace("-", ""));
		}
		log.info("-=======>>>>SQL:" + queryWrapper.getCustomSqlSegment());
		return queryWrapper;
	}

	/**
	 * 根据规则获取人群查询条件
	 * 
	 * @param ruleJson
	 * @return
	 */
	public QueryWrapper<ZUserTagEntity> zUserTagParseRule(JSONObject ruleJson) {
		JSONArray ruleColums = ruleJson.getJSONArray("ruledetails");
		String relationship = ruleJson.getString("relationship");

		QueryWrapper<ZUserTagEntity> queryWrapper = Wrappers.query();
		if (ruleColums.size() > 0) {
			for (int i = 0; i < ruleColums.size(); i++) {
				String key = ruleColums.getJSONObject(i).getString("key");
				String value = ruleColums.getJSONObject(i).getString("value");
				JSONArray jsonArr = JSONArray.parseArray(value);
				if (jsonArr.size() > 0) {
					queryWrapper = getQueryWrapper(relationship, (String) jsonArr.get(0), key, jsonArr, queryWrapper);
				}
			}
		}
		// 判断是否有日期
		JSONArray dateRangeArr = ruleJson.getJSONArray("daterange");
		if (dateRangeArr != null && dateRangeArr.size() == 2) {
			queryWrapper.ge("THEMONTH_V", dateRangeArr.getString(0).replace("-", "")).le("THEMONTH_V",
					dateRangeArr.getString(1).replace("-", ""));
		}
		return queryWrapper;
	}

	/**
	 * 获取ZUserTagEntity 的查询
	 * 
	 * @param relationship
	 * @param tagCode
	 * @param jsonArr
	 * @param queryWrapper
	 * @return
	 */
	public QueryWrapper<ZUserTagEntity> getQueryWrapper(String relationship, String tagCode, String tagName,
			JSONArray jsonArr, QueryWrapper<ZUserTagEntity> queryWrapper) {
		// 根据标签code 获取列信息
		TagColumn whereCol = userProfileService.getColumName(tagCode, tagName);
		if(whereCol == null) {
			return queryWrapper;
		}
		// 且
		if (relationship.equals("and")) {
			if (whereCol.columnType.equals("array")) {
				String arryValue = "[" + jsonArr.stream().map(s -> "\'" + s + "\'").collect(Collectors.joining(","))
						+ "]";
				queryWrapper.and(a -> a.apply("hasAny(`" + whereCol.columnE.toUpperCase() + "`, " + arryValue + ")"));
			} else if (whereCol.columnType.equals("string")) {
				if ("event_unspname".equalsIgnoreCase(whereCol.englishName)) {
					queryWrapper.and(a -> a.notIn("EVENT_SPNAME", jsonArr.toArray()));
				} else {
					queryWrapper.and(a -> a.in(whereCol.columnE.toUpperCase(), jsonArr.toArray()));
				}
			}
			// 或
		} else {
			if (whereCol.columnType.equals("array")) {
				String arryValue = "[" + jsonArr.stream().map(s -> "\'" + s + "\'").collect(Collectors.joining(","))
						+ "]";
				queryWrapper.or(a -> a.apply("hasAny(`" + whereCol.columnE.toUpperCase() + "`," + arryValue + ")"));
			} else if (whereCol.columnType.equals("string")) {
				if ("event_unspname".equalsIgnoreCase(whereCol.englishName)) { // 未订购标签做特殊处理
					queryWrapper.or(a -> a.notIn("EVENT_SPNAME", jsonArr.toArray()));
				} else {
					queryWrapper.or(a -> a.in(whereCol.columnE.toUpperCase(), jsonArr.toArray()));
				}
			}
		}
		return queryWrapper;
	}

	/**
	 * 主函数
	 */
	public JSONObject usergroupCalculate(UserGroupRequest userGroupRequest) {
		// 圈人规则,日期参数,是否导出文件
		JSONObject result = new JSONObject();
		Integer userCount = parseRule(userGroupRequest.getRuleinfo(), userGroupRequest.getIssave());
		result.put("total", userCount);
		return result;
	}

	public JSONObject usergroupCalculateBaseImport(UserGroupRequest userGroupRequest) {
		// 圈人规则,日期参数,是否导出文件
		JSONObject result = new JSONObject();
		Integer userCount = parseRuleBaseImport(userGroupRequest.getImportId(), userGroupRequest.getRuleinfo(),
				userGroupRequest.getIssave());
		result.put("total", userCount);
		return result;
	}

	@Override
	public List<String> queryUserIdList(UserGroupEntity userGroupEntity, int offset, int pageSize) {
		String sqlLimit = "limit " + offset + "," + pageSize;
		String importId = userGroupEntity != null ? userGroupEntity.getImportId() : null;
		JSONObject rule = JSON.parseObject(userGroupEntity.getRules());
		if(StringUtils.isBlank(importId)) {
			//普通人群
			QueryWrapper<ZUserTagEntity> queryWrapper = zUserTagParseRule(rule);
			queryWrapper.last(sqlLimit);
			return userTagsMapper.queryUserIdList(queryWrapper);
		}else {
			//基于导入的人群
			QueryWrapper<ZUserTagEntity> childWrapper = zUserTagParseRuleExport(rule);
			return  userTagsMapper.queryUserIdListBaseImport(childWrapper, importId, sqlLimit);
			
//			return  userTagsMapper.queryUserIdListAllImport(importId);
		}
	}
	
	/**
	 * 
	 */
	@Override
	public List<String> queryUserIdListFromImport(String importId,int offset,int pageSize){
		String sqlLimit = "limit " + offset + "," + pageSize;
		return  userTagsMapper.queryUserIdListFromImport(importId,sqlLimit);
	}

}
