package org.springblade.useranalysis.service.normal;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.springblade.useranalysis.common.utils.HashKit;
import org.springblade.useranalysis.common.utils.JsonMap;
import org.springblade.useranalysis.dto.param.ExcelExportParam;
import org.springblade.useranalysis.entity.TagColumn;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.entity.UserProfileExcelLog;
import org.springblade.useranalysis.entity.clickhouse.ZUserTagEntity;
import org.springblade.useranalysis.service.UserGroupService;
import org.springblade.useranalysis.service.UserProfileService;
import org.springblade.useranalysis.service.UserTagsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.converters.longconverter.LongStringConverter;
import com.alibaba.excel.util.FileUtils;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.xiaoymin.knife4j.core.util.StrUtil;

import lombok.extern.slf4j.Slf4j;

/**
 * @author william
 *
 */
@Slf4j
@Service
public class ExcelService {

	private final static int PAGE_SIZE = 50000;

	@Autowired
	private UserProfileExcelLogService excelLogService;

	@Autowired
	private UserTagsService userTagsService;

	@Autowired
	private UserProfileService userProfileService;

	@Autowired
	private UserGroupService userGroupService;

	final ExecutorService threadPool = new ThreadPoolExecutor(1, 2, 5, TimeUnit.MINUTES,
			// 有界队列5000，超过队列长度，无异常抛弃
			new ArrayBlockingQueue<>(5000), new ThreadFactory() {
				@Override
				public Thread newThread(Runnable r) {
					Thread thread = new Thread(r);
					thread.setDaemon(false);
					thread.setName("AdsMarket-Message-Pool-");
					return thread;
				}
			}, new ThreadPoolExecutor.DiscardPolicy());

	public void excute(ExcelExportParam params, QueryWrapper<ZUserTagEntity> childWrapper,
			UserProfileExcelLog excelLog) {
		threadPool.execute(() -> {
			log.info(excelLog.getId() + " - exporting......");
			long start = System.currentTimeMillis();
			try {
				boolean isSuccess = handle(params, childWrapper, excelLog);
				if (isSuccess) {
					excelLog.setCount(params.getCount());
					excelLog.setStatus(UserProfileExcelLogService.STATUS_SUCCESS);
					excelLogService.saveOrUpdate(excelLog);
					log.info(excelLog.getId() + " - export success");
				} else {
					log.info(excelLog.getId() + " - export failure");
					deleteCache(excelLog);
				}
			} catch (Exception e) {
				deleteCache(excelLog);
				log.info(excelLog.getId() + " - export failure,the error is ：" + e.getMessage());
			}
			long end = System.currentTimeMillis();
			log.info(excelLog.getId() + " - take up time is " + (end - start) / 1000.f + "second");
		});
	}

	private void deleteCache(UserProfileExcelLog excelLog) {
		try {
			// 导出失败
			excelLog.setStatus(UserProfileExcelLogService.STATUS_FAILURE);
			excelLogService.saveOrUpdate(excelLog);
			// 删除导出失败的垃圾文件
			FileUtils.delete(new File(excelLog.getFilePath()));
		} catch (Exception e2) {
			log.info(excelLog.getId() + " - delete the file that export failure,the error is " + e2.getMessage());
		}
	}

	private boolean handle(ExcelExportParam params, QueryWrapper<ZUserTagEntity> childWrapper,
			UserProfileExcelLog excelLog) {
		boolean isSuccess = true;
		List<String> codeList = params.getTagList();
		boolean desens = params.isDesens();
		StringBuffer fields = new StringBuffer();
		List<TagColumn> columnList = getFieldList(codeList);
		columnList.forEach(col -> {
			if (StrUtil.isNotBlank(col.columnV) && !col.columnV.contains("USERID")) {
				if (StrUtil.isNotBlank(col.showHistory) && col.showHistory.equalsIgnoreCase("1")) {
					fields.append(col.columnV);
				} else {
					fields.append(col.columnC);
				}
				fields.append(",");
			}
		});
		fields.append("USERID_V,THEMONTH_V");
		File excelFile = new File(excelLog.getFilePath());
		ExcelWriter excelWriter = EasyExcel.write(excelFile).registerConverter(new LongStringConverter())
				.head(buidlHeader(columnList)).build();
		UserGroupEntity group = userGroupService.getById(params.getGroupId());
		String importId = group != null ? group.getImportId() : null;
		// 需要导出的数量
		int count = params.getCount();
		// 已经查寻的数量
		int total = 0;
		// 待查询数量
		int rest = count - total;
		// 下次查询偏移量
		int offset = 0;

		while (true) {
			int pageSize = PAGE_SIZE;
			if (count <= PAGE_SIZE) {
				// 如果不足一万条，一次性查完
				pageSize = count;
			}
			if (rest <= PAGE_SIZE) {
				// 带查询的数量小于等于一万条，一次性查完
				pageSize = rest;
			}

			String sqlLimit = "limit " + offset + "," + pageSize;
			List<Map<String, Object>> tagList = null;
			try {
				if (StringUtils.isNotBlank(params.getMonthStart()) && StringUtils.isNotBlank(params.getMonthEnd())) {
					tagList = userTagsService.exportTagListMonths(fields.toString(), params.getMonthStart(),
							params.getMonthEnd(), childWrapper, sqlLimit);
				} else {
					if (StringUtils.isBlank(importId)) {
						childWrapper.last(sqlLimit);
					}
					tagList = userTagsService.exportTagList(fields.toString(), childWrapper, importId, sqlLimit);
				}
			} catch (Exception e) {
				log.error("ExcelService handle fail, the error is " + e.getMessage());
				isSuccess = false;
				break;
			}

			if (tagList == null || tagList.isEmpty()) {
				// 没有数据
				break;
			}

			total += tagList.size();
			rest = count - total;
			excel(desens, excelWriter, columnList, tagList, excelLog.getGroupName());
			if (total == count || tagList.size() < pageSize) {
				// 当前已经是最后一页
				break;
			}

			offset += pageSize;
		}

		if (excelWriter != null) {
			excelWriter.finish();
		}

		params.setCount(total);
		return isSuccess;
	}

	/**
	 * 动态生成表头
	 */
	private List<List<String>> buidlHeader(List<TagColumn> columnList) {
		List<List<String>> list = new ArrayList<List<String>>();

		List<String> headPhone = new ArrayList<String>();
		headPhone.add("手机号");
		list.add(headPhone);

		List<String> headMonth = new ArrayList<String>();
		headMonth.add("月份");
		list.add(headMonth);

		columnList.forEach(col -> {
			List<String> head = new ArrayList<String>();
			head.add(col.cnName);
			list.add(head);
		});
		return list;
	}

	private List<TagColumn> getFieldList(List<String> codeList) {
		List<TagColumn> list = new ArrayList<>();
		if (codeList != null && !codeList.isEmpty()) {
			codeList.forEach(code -> {
				TagColumn weiduColumn = userProfileService.getColumNameExport(code);
				if (weiduColumn != null && weiduColumn.columnC != null
						&& !weiduColumn.columnC.contains("EVENT_UNSPNAME")) {
					// 该字段会严重影响查询速度，暂时屏蔽掉
					list.add(weiduColumn);
				}
			});
		}
		return list;
	}

	private void excel(boolean desens, ExcelWriter excelWriter, List<TagColumn> columnList,
			List<Map<String, Object>> tagList, String groupName) {
		groupName = StringUtils.isNotBlank(groupName) ? groupName : "1";
		List<List<Object>> list = new ArrayList<List<Object>>();
		Map<String, Object> mobileMap = new HashMap<>();
		tagList.forEach(tag -> {
			List<Object> data = new ArrayList<Object>();
			/** ---------去重手机号------ **/
			String mobile = tag.get("USERID_V").toString();
			if(mobileMap.containsKey(mobile)) {
				return;
			}
			mobileMap.put(mobile, new Object());
			/** ---------去重手机号------ **/
			if (desens) {
				// 手机号脱敏
				data.add(HashKit.sha1(tag.get("USERID_V").toString()));
			} else {
				data.add(tag.get("USERID_V"));
			}
			Object month = tag.get("THEMONTH_V");
			data.add(month != null ? month.toString() : "");

			columnList.forEach(col -> {
				Object obj = tag.get(col.columnC);
				if (StrUtil.isNotBlank(col.showHistory) && col.showHistory.equalsIgnoreCase("1")) {
					obj = tag.get(col.columnV);
				}
				if (StrUtil.isNotBlank(col.columnType) && col.columnType.equalsIgnoreCase("array")) {
					String value = "NaN";
					JsonMap jsonMap = JsonMap.parseJson(JSON.toJSONString(tag));
					String[] array = col.columnC.split("\\.");
					if (array != null && array.length == 2 && jsonMap != null) {
						String arrayParent = array[0];
						String arrayChild = array[1];
						JsonMap parent = jsonMap.getMap(arrayParent);
						if (parent != null) {
							List<String> childList = parent.getStringList(arrayChild);
							if (childList != null && !childList.isEmpty()) {
								value = JsonMap.toJson(childList);
							}
						}
					} else {
						value = obj != null ? obj.toString() : "NaN";
					}
					data.add(value);
				} else {
					data.add(obj != null ? obj.toString() : "NaN");
				}
			});

			list.add(data);
		});

		excelWriter.write(list, EasyExcel.writerSheet(groupName).build());
	}
}
